@@ -31,7 +31,8 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'riece-message))
+(require 'riece-message)
+(require 'riece-commands)
 
 (defvar riece-unread-channels nil)
 
@@ -59,7 +60,8 @@
 	  (goto-char (point-min))
 	  (while (not (eobp))
 	    (if (looking-at "\\( ?[0-9]+:\\)\\([ !]\\)\\(.+\\)")
-		(let ((channel (match-string 3)))
+		(let ((channel (save-match-data
+				 (riece-parse-identity (match-string 3)))))
 		  (replace-match
 		   (concat "\\1"
 			   (if (member channel riece-unread-channels)
@@ -83,8 +85,8 @@
 	    'riece-unread-display-message-function)
   (add-hook 'riece-channel-switch-hook
 	    'riece-unread-channel-switch-hook)
-  (add-hook 'riece-update-buffers-hook
-	    'riece-unread-update-channel-list-buffer)
+  (add-hook 'riece-update-buffer-functions
+	    'riece-unread-update-channel-list-buffer t)
   (define-key riece-command-mode-map
     "\C-c\C-u" 'riece-unread-switch-to-channel)
   (define-key riece-dialogue-mode-map
