@@ -23,8 +23,6 @@
 
 ;;; Code:
 
-(require 'riece-compat)
-
 (defun riece-xemacs-hide-modeline ()
   "Remove modeline from current window."
   (set-specifier has-modeline-p nil (current-buffer)))
@@ -70,6 +68,9 @@ Modify whole identification by side effect."
 (defalias 'riece-simplify-mode-line-format
   'riece-xemacs-simplify-modeline-format)
 
+(defalias 'riece-set-case-syntax-pair
+  'put-case-table-pair)
+
 (provide 'riece-xemacs)
 
 ;;; riece-xemacs.el ends here
